package deployer

import (
	"bytes"
	"fmt"
	"io"
	"os"
	"os/exec"
	"strings"
)

func Cmd(cmdnarg ...string) *exec.Cmd {
	cmd := exec.Command(cmdnarg[0], cmdnarg[1:]...)
	cmd.Stdin = os.Stdin
	cmd.Stdout = os.Stdout
	cmd.Stderr = os.Stderr
	return cmd
}

func Exec(workDir string, cmds ...*exec.Cmd) {
	ThrowIfErr(shellcmd(true, workDir, os.Stdout, cmds))
}

func ExecAllowingErr(workDir string, cmds ...*exec.Cmd) {
	_ = shellcmd(true, workDir, os.Stdout, cmds)
}

func ExecStr(workDir string, cmds ...*exec.Cmd) string {
	var stdout bytes.Buffer
	ThrowIfErr(shellcmd(false, workDir, &stdout, cmds))
	return string(stdout.Bytes())
}

func shellcmd(debug bool, workDir string, stdout io.Writer, cmds []*exec.Cmd) error {
	if debug {
		// Print the commands for output debug
		var cmdlines []string
		for _, cmd := range cmds {
			cmdStr := fmt.Sprintf("%v", cmd.Args)
			cmdlines = append(cmdlines, fmt.Sprintf("%s", cmdStr[1:len(cmdStr)-1]))
		}
		Echof("$ %s", strings.Join(cmdlines, " | "))
	}

	if len(cmds) == 0 {
		return fmt.Errorf("Shell called without any Cmd")
	}

	// Chain the commands via pipes
	var (
		p   io.Reader = os.Stdin
		err error
	)
	for _, cmd := range cmds[:len(cmds)-1] {
		cmd.Stdin = p
		p, err = cmd.StdoutPipe()
		ThrowIfErr(err)
	}
	cmds[len(cmds)-1].Stdin = p
	cmds[len(cmds)-1].Stdout = stdout

	for _, cmd := range cmds {
		cmd.Dir = workDir
		if err := cmd.Start(); err != nil {
			return err
		}
	}

	for i := len(cmds) - 1; i >= 0; i-- {
		if err := cmds[i].Wait(); err != nil {
			return err
		}
	}

	return nil
}

func Echof(format string, args ...interface{}) {
	fmt.Printf(format+"\n", args...)
}

func Echo(s string) {
	fmt.Errorf("%s\n", s)
}
